@@ -26,7 +26,8 @@ single_trial_po = function(or, pro_ctr, n, cf, threshold, method){
     result_string = "early stop due to the futility"
   }else{
     #get the rest of data  into trial
-    dat_II = data_gene(or, sd, pro_ctr, n)
+    #dat_II = data_gene(or, sd, pro_ctr, n)
+    dat_II = dat_I
     N = N + n
     #combine two stages data
     dat = rbind(dat_I, dat_II)
@@ -58,9 +59,9 @@ multiple_trial_po = function(sim_runs, or, pro_ctr, n, cf, threshold, method){
   preout = apply(or, MARGIN = 1, FUN=single_trial_po,
                  pro_ctr=pro_ctr, n=n, cf=cf, threshold=threshold,
                  method = method)
-  prob = mean(preout[1,] == "success")
-  prob_et = 100*mean(preout[1,] == "early stop due to the futility")
-  avgss = mean(as.numeric(preout[2,]))
+  prob = round(mean(preout[1,] == "success"), digits = 4)
+  prob_et = round(100*mean(preout[1,] == "early stop due to the futility"), digits = 2)
+  avgss = round(mean(as.numeric(preout[2,])), digits = 0)
   return(c(prob_et, prob, avgss))
 }
 
@@ -151,20 +152,20 @@ multiple_trial_npo = function(or.mat, sd, pro_ctr, U, n, cf, threshold, method){
                  method = method)
   # remove those results with warning messages
   preout_filter = preout[,which(preout[3,] == "No" & preout[4,] == "No")]
-
-  prob = mean(preout_filter[1,] == "success")
-
-  prob_et = 100*mean(preout_filter[1,] == "early stop due to the futility")
-  avgss = mean(as.numeric(preout_filter[2,]))
+  prob = round(mean(preout_filter[1,] == "success"),digits = 4)
+  prob_et = round(100*mean(preout_filter[1,] == "early stop due to the futility"),digits = 2)
+  avgss = round(mean(as.numeric(preout_filter[2,])),digits = 0)
   return(c(prob_et, prob, avgss))
 }
 
 
 
+# 3rd design - switch model
 # 3rd design - switch model
 single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
                                alpha, power, method){
 
+  model_sele = NA;
   #choose larger sample size for stage 1
   if(n_po >= n_npo){
     n = n_po
@@ -173,7 +174,9 @@ single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
   C  = length(pro_ctr)
   if(all(or == or[1])){
     dat_I = data_gene(or, sd, pro_ctr, n)
-  }else{dat_I = data_gene_npo(or, sd, pro_ctr, n)}
+  }else{
+    dat_I = data_gene_npo(or, sd, pro_ctr, n)
+  }
 
   N = n
   #1. Stage I select PO or NPO through MCMC
@@ -183,16 +186,17 @@ single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
 
   #2. Estimate parameters based on the model selection result (PO/NPO)
   if(prob[1] > prob[2]){
+    model_sele = 'PO'
     results = jags_po_model(dat_I)
     post_draws_I = results$Samples[[1]][,1]
     pr_trt_futility_I = mean(post_draws_I < 0)
   }else{
+    model_sele = 'NPO'
     results = jags_npo_model(dat_I, U)
     pr_trt_futility_I = results$PPUtilities
   }
 
   stop_for_futility = (pr_trt_futility_I <= cf)
-
   warnings = "No"
 
   #3. Decide if we go/no go to stage 2
@@ -201,7 +205,7 @@ single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
   }else{
     #4. Re-estimate sample size based on the model selection result
     if(prob[1] > prob[2]){
-      cat("PO model is selected.")
+
       dat_II = data_gene(or, sd, pro_ctr, n_po)
       N = N + n_po
       dat = rbind(dat_I, dat_II)
@@ -218,11 +222,129 @@ single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
       }
     }
     if(prob[2] > prob[1]){
-      cat("NPO model is selected.")
+
       dat_II = dat_I
       #dat_II = data_gene_npo(or, sd, pro_ctr, n_npo)
       N = N + n_npo
       dat = rbind(dat_I, dat_II)
+      if(method == "Bayesian"){
+        results = jags_npo_model(dat, U)
+        pr_trt_futility_II = results$PPUtilities
+      }else if(method == "Frequentist"){
+        message = tryCatch(ordinal::clm(response ~ treatment, nominal = ~ treatment,
+                                        data=dat, threshold = "flexible"),
+                           error=function(e) e, warning=function(w) w)
+        fit = ordinal::clm(response ~ treatment, nominal = ~ treatment,
+                           data=dat, threshold = "flexible")
+        #utility function
+        u = utility_func(fit,  U, C)
+        #obtain utility from proportions
+        pr_trt_futility_II = mean(u[2,] > u[1,])
+        # check if warning message exists
+        if(is(message,"warning")) {
+          warnings = "Yes"
+        }else{warnings = "No"}
+      }
+    }
+
+    if(pr_trt_futility_II >= threshold){
+      result_string = "success"
+    }else if(pr_trt_futility_II < threshold)
+    {result_string = "failure"}
+  }
+  output = c(result_string, N, warnings, model_sele)
+  return(output)
+}
+
+
+multiple_trial_switch = function(or, sim_runs, sd, pro_ctr, n_po, n_npo,
+                                 U, cf, threshold, method){
+  #nmax: maximum sample size used for type I error control
+  #n: sample size for first stage
+  if(all(or == or[1])){
+    #random generate or
+    ors = exp(rnorm(sim_runs, mean = log(or[1]), sd = sd))
+    or.mat = matrix(rep(ors, times = 1, each=length(or)), nrow = sim_runs,
+                    ncol = length(or), byrow = TRUE)
+  }else{
+    #directly put into func
+    or.mat = matrix(rep(or, sim_runs), nrow = sim_runs,
+                    ncol = length(or), byrow = TRUE)
+  }
+  preout = apply(or.mat, MARGIN = 1, FUN = single_trial_switch, sd=sd,
+                 pro_ctr, n_po, n_npo, U, cf = cf, threshold=threshold,
+                 method = method)
+  preout_filter = preout[,which(preout[3,] == "No")]
+  prob = mean(preout_filter[1,] == "success")
+  prob_et = 100*mean(preout_filter[1,] == "early stop due to the futility")
+  avgss = mean(as.numeric(preout_filter[2,]))
+  #m = table(factor(preout_filter[4,], levels = c('PO', 'NPO')))/dim(preout_filter[,which(preout_filter[1,]!= 'early stop due to the futility')])[2]*100
+  m = table(factor(preout_filter[4,], levels = c('PO', 'NPO')))/dim(preout_filter)[2]*100
+  return(c(prob_et, prob, avgss, m))
+}
+
+
+single_trial_switch_new = function(or, sd, pro_ctr, n, U, cf, threshold,
+                               alpha, power, method){
+
+  model_sele = NA;
+
+  C  = length(pro_ctr)
+  if(all(or == or[1])){
+    dat_I = data_gene(or, sd, pro_ctr, n)
+  }else{
+    dat_I = data_gene_npo(or, sd, pro_ctr, n)
+    }
+
+  N = n
+  #1. Stage I select PO or NPO through MCMC
+  result_rjmcmc = rjmcmc_inter(or, sd, pro_ctr, n, U)
+  prob = result_rjmcmc$result$`Posterior Model Probabilities`
+  #print(prob)
+
+  #2. Estimate parameters based on the model selection result (PO/NPO)
+  if(prob[1] > prob[2]){
+    model_sele = 'PO'
+    results = jags_po_model(dat_I)
+    post_draws_I = results$Samples[[1]][,1]
+    pr_trt_futility_I = mean(post_draws_I < 0)
+  }else{
+    model_sele = 'NPO'
+    results = jags_npo_model(dat_I, U)
+    pr_trt_futility_I = results$PPUtilities
+  }
+
+  stop_for_futility = (pr_trt_futility_I <= cf)
+  warnings = "No"
+
+  #3. Decide if we go/no go to stage 2
+  if(stop_for_futility){
+    result_string = "early stop due to the futility"
+  }else{
+    #4. Re-estimate sample size based on the model selection result
+    if(prob[1] > prob[2]){
+
+      dat_II = data_gene(or, sd, pro_ctr, n)
+      N = N + n
+      dat = rbind(dat_I, dat_II)
+      if(method == "Bayesian"){
+        results = jags_po_model(dat)
+        post_draws_II = results$Samples[[1]][,1]
+        pr_trt_futility_II = mean(post_draws_II < 0)
+      }else if(method == "Frequentist"){
+        fit.prior = ordinal::clm(response ~ treatment, data=dat, threshold = "flexible")
+        mean.fit = as.numeric(fit.prior$beta)
+        sd.fit = sqrt(fit.prior$vcov[C, C])
+        samples_draws_II = rnorm(10000, mean.fit, sd.fit)
+        pr_trt_futility_II = mean(samples_draws_II < 0)
+      }
+    }
+    if(prob[2] > prob[1]){
+
+      dat_II = dat_I
+      #dat_II = data_gene_npo(or, sd, pro_ctr, n_npo)
+      N = N + n
+      dat = rbind(dat_I, dat_II)
       if(method == "Bayesian"){
         results = jags_npo_model(dat, U)
         pr_trt_futility_II = results$PPUtilities
@@ -248,12 +370,12 @@ single_trial_switch = function(or, sd, pro_ctr, n_po, n_npo, U, cf, threshold,
     }else if(pr_trt_futility_II < threshold)
     {result_string = "failure"}
   }
-  output = c(result_string, N, warnings)
+  output = c(result_string, N, warnings, model_sele)
   return(output)
 }
 
 
-multiple_trial_switch = function(or, sim_runs, sd, pro_ctr, n_po, n_npo,
+multiple_trial_switch_new = function(or, sim_runs, sd, pro_ctr, n,
                                  U, cf, threshold, method){
   #nmax: maximum sample size used for type I error control
   #n: sample size for first stage
@@ -268,13 +390,14 @@ multiple_trial_switch = function(or, sim_runs, sd, pro_ctr, n_po, n_npo,
                     ncol = length(or), byrow = TRUE)
   }
   preout = apply(or.mat, MARGIN = 1, FUN = single_trial_switch, sd=sd,
-                 pro_ctr, n_po, n_npo, U, cf = cf, threshold=threshold,
+                 pro_ctr, n, U, cf = cf, threshold=threshold,
                  method = method)
   preout_filter = preout[,which(preout[3,] == "No")]
-  prob = mean(preout_filter[1,] == "success")
-  prob_et = 100*mean(preout_filter[1,] == "early stop due to the futility")
-  avgss = mean(as.numeric(preout_filter[2,]))
-  return(c(prob_et, prob, avgss))
+  prob = round(mean(preout_filter[1,] == "success"), digits = 4)
+  prob_et = round(100*mean(preout_filter[1,] == "early stop due to the futility"), digits = 2)
+  avgss = round(mean(as.numeric(preout_filter[2,])), digits = 0)
+  m = table(factor(preout_filter[4,], levels = c('PO', 'NPO')))/dim(preout_filter[,which(preout_filter[1,]!= 'early stop due to the futility')])[2]*100
+  return(c(prob_et, prob, avgss, m))
 }
 
 
